<?php
/*
Plugin Name: Auto Code
Description: 自动格式化代码片段并支持语法高亮
*/

// 注册短代码
add_shortcode('code', 'auto_code_formatter_shortcode');

// 内容过滤器
add_filter('the_content', 'auto_detect_code_blocks', 9);
add_filter('content_save_pre', 'auto_format_code_blocks', 5);

// 加载前端资源
add_action('wp_enqueue_scripts', 'auto_code_formatter_assets');

function auto_code_formatter_assets() {
    // Prism.js官方CDN资源（使用jsDelivr）
    wp_enqueue_style(
        'prism-css',
        'https://cdn.jsdelivr.net/npm/prismjs@1.29.0/themes/prism.min.css',
        array(),
        '1.29.0'
    );
    
    wp_enqueue_script(
        'prism-core',
        'https://cdn.jsdelivr.net/npm/prismjs@1.29.0/components/prism-core.min.js',
        array(),
        '1.29.0',
        true
    );

    wp_enqueue_script(
        'prism-autoloader',
        'https://cdn.jsdelivr.net/npm/prismjs@1.29.0/plugins/autoloader/prism-autoloader.min.js',
        array('prism-core'),
        '1.29.0',
        true
    );
}

// 短代码处理函数
function auto_code_formatter_shortcode($atts, $content = null) {
    $atts = shortcode_atts(array(
        'lang' => 'auto'
    ), $atts);

    $language = sanitize_text_field($atts['lang']);
    $content = htmlspecialchars(trim($content), ENT_NOQUOTES);

    // 自动检测语言
    if ($language === 'auto') {
        $language = detect_code_language($content);
    }

    return '<pre class="language-'.esc_attr($language).'"><code class="language-'.esc_attr($language).'">'.$content.'</code></pre>';
}

// 改进版语言检测
function detect_code_language($code) {
    $code = trim($code);
    
    $detectors = array(
        'php' => ['<?php', '->', '::', 'namespace '],
        'javascript' => ['function(', '=>', 'let ', 'const ', 'export default'],
        'css' => ['@import', '{', '}', ';'],
        'python' => ['def ', 'import ', 'from ', 'lambda:'],
        'html' => ['<!DOCTYPE', '<html', '<head>', '<div ']
    );

    foreach ($detectors as $lang => $patterns) {
        foreach ($patterns as $pattern) {
            if (strpos($code, $pattern) !== false) {
                return $lang;
            }
        }
    }

    // 通过特殊符号检测
    if (strpos($code, '<?') === 0) return 'php';
    if (strpos($code, '<!DOCTYPE') === 0) return 'html';
    if (preg_match('/^[\w-]+:\s+/m', $code)) return 'yaml';

    return 'generic';
}

// 自动格式化代码块
function auto_format_code_blocks($content) {
    // 转换Markdown风格代码块
    $content = preg_replace_callback(
        '/```([\w-]*)\s*([\s\S]*?)```/',
        function($matches) {
            $lang = $matches[1] ?: 'auto';
            return '[code lang="'.$lang.'"]'.trim($matches[2]).'[/code]';
        },
        $content
    );

    // 转换未格式化的pre标签
    $content = preg_replace_callback(
        '/<pre>(.*?)<\/pre>/is',
        function($matches) {
            $code = strip_tags(trim($matches[1]));
            return '[code lang="auto"]'.$code.'[/code]';
        },
        $content
    );

    return $content;
}

// 自动处理现有代码块
function auto_detect_code_blocks($content) {
    return preg_replace_callback(
        '/<code>(.*?)<\/code>/is',
        function($matches) {
            $code = htmlspecialchars_decode($matches[1]);
            return do_shortcode('[code lang="auto"]'.$code.'[/code]');
        },
        $content
    );
}

// TinyMCE编辑器集成
add_action('admin_init', 'auto_code_formatter_editor');
function auto_code_formatter_editor() {
    if (!current_user_can('edit_posts') && !current_user_can('edit_pages')) {
        return;
    }

    if (get_user_option('rich_editing')) {
        add_filter('mce_external_plugins', 'auto_code_formatter_tinymce_plugin');
        add_filter('mce_buttons', 'auto_code_formatter_tinymce_button');
    }
}

function auto_code_formatter_tinymce_plugin($plugin_array) {
    $plugin_array['codeformatter'] = plugins_url('tinymce-plugin.js', __FILE__);
    return $plugin_array;
}

function auto_code_formatter_tinymce_button($buttons) {
    array_push($buttons, 'codeformatter');
    return $buttons;
}